let waitingMessageDiv = document.getElementById("waitingMessageDiv");
let endMessageDiv = document.getElementById("endMessageDiv");
let gameDiv = document.getElementById("gameDiv");
let wordChoiceDiv = document.getElementById("wordChoiceDiv");

let roundSpan = document.getElementById("roundSpan");
let playerList = document.getElementById("playerList");
let wordSpan = document.getElementById("wordSpan");
let timerSpan = document.getElementById("timerSpan");
let canvas = document.getElementById("drawingCanvas");
let context = canvas.getContext("2d");

context.lineJoin = context.lineCap = "round";

//TODO: Get our own cursor
let drawingCursor = 'url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAApUlEQVRYR+2TwQ2AIBAEoQ6asAYr4GMRVmQRfqjAGmyCPgQjL59zJpe4fOCzl8ncEoPzE53zBQHSDcmgDFIDNK8OyiA1QPPqoAxSAzT/uw4uj7Gdmht5S4NHSumeW2vt12wBaQW4NLi1lDJ1qJzz2SC39sQmfwPYxble8aib609i8SdeM6w6+AlcHypAqlYGZZAaoHl1UAapAZpXB2WQGqB59x28AOjbHClHA3hIAAAAAElFTkSuQmCC") 20 20, default';

let drawingToolbox = document.getElementById("drawingToolbox");

let eraserToggle = document.getElementById("eraserToggle");
let clearButton = document.getElementById("clearButton");
let toolSizeSelect = document.getElementById("toolSizeSelect");
let colourSelect = document.getElementById("colourSelect");

let guessingBox = document.getElementById("guessingBox");
let guessInput = document.getElementById("guessInput");

let firstWordButton = document.getElementById("firstWordButton");
let secondWordButton = document.getElementById("secondWordButton");
let thirdWordButton = document.getElementById("thirdWordButton");

let role = null;
let gameStarted = false;
let gameEnded = false;
let drawer = null;
let players = [];
let wordFinders = [];
let wordOptions = [];
let timerId = null;

let drawing = false;
let points = [];
let brush = { colour: "#000000", size: 5 };
let erasing = false;

let startPoint = endPoint = null;

let webSocket = null;

function init()
{
	setRole("waiter");

	webSocket = new WebSocket("ws://" + window.location.host + "/ws/game/" + lobbyId);
	webSocket.onmessage = function(e) { onReceiveMessage(e.data); }
	webSocket.onopen = function() { sendMessage("joinGame", nickname); };

	window.onbeforeunload = function() {
		if (webSocket.readyState == WebSocket.OPEN)
		{
			sendMessage("leaveGame", null);
		}
	};
}

function sendMessage(type, data)
{
	webSocket.send(JSON.stringify({ type: type, data: data }));
}

function onReceiveMessage(messageText)
{
	let message = JSON.parse(messageText);

	switch(message.type)
	{
		case "setGameState":
			if(message.data.ended)
			{
				setRole("waiter");

				gameDiv.style.display = "none";
				endMessageDiv.style.display = "";
				return;
			}

			if(message.data.drawer == nickname && message.data.word != "")
			{
				setRole("drawer");

				wordChoiceDiv.style.display = "none";
			}
			else if(message.data.wordFinders.includes(nickname))
			{
				setRole("wordFinder");
			}
			else
			{
				setRole("guesser");
			}

			timerSpan.innerText = message.data.secondsLeft;

			if(message.data.drawer != drawer && message.data.word != "")
			{
				drawer = message.data.drawer;

				context.clearRect(0, 0, canvas.width, canvas.height);

				addNewDrawerElement(drawer);

				startTimer();
			}

			roundSpan.innerText = message.data.currentRound;
			wordSpan.innerText = message.data.word;

			playerList.innerHTML = "";
			for(player of message.data.players)
			{
				//TODO: Mark word finders, mark this player
				let playerItem = document.createElement("li");
				playerItem.innerText = player;
				if(player == nickname)
				{
					playerItem.innerText += " (You)";
				}
				if(player == message.data.drawer)
				{
					playerItem.innerText += " ✏️";
				}
				if(message.data.wordFinders.includes(player))
				{
					playerItem.style.color = "green";
				}
				playerList.appendChild(playerItem);

				if(!players.includes(player))
				{
					addPlayerJoinElement(player);
				}
			}
			for(player of players)
			{
				if(!message.data.players.includes(player))
				{
					addPlayerLeaveElement(player);
				}
			}
			players = message.data.players;

			for(wordFinder of message.data.wordFinders)
			{
				if(wordFinders.includes(wordFinder))
				{
					continue;
				}

				addNewWordFinderElement(wordFinder);
			}
			wordFinders = message.data.wordFinders;

			waitingMessageDiv.style.display = "none";
			gameDiv.style.display = "";

			gameStarted = true;
			break;
		case "addGuess":
			addGuessElement(message.data.guesser, message.data.guess);
			break;
		case "addChatMessage":
			addChatMessageElement(message.data.sender, message.data.text);
			break;
		case "revealWord":
			revealWord(message.data);
			break;
		case "setChoosingPlayer":
			setChoosingPlayer(message.data);
			break;
		case "setWordOptions":
			setWordOptions(message.data);
			break;
		case "youAreAlone":
			let element = document.createElement("p");
			element.style.marginTop = "0";
			element.style.marginBottom = "0";
			element.style.color = "red";
			element.innerHTML = "You are the only player left, returning to lobby...";
			guessingBox.appendChild(element);

			setTimeout(function() {
				window.location.href = "/lobby/" + lobbyId;
			}, 3000);
			break;
		case "setStartPoint":
			setStartPoint(message.data.x, message.data.y);
			break;
		case "setEndPoint":
			setEndPoint(message.data.x, message.data.y);
			break;
		case "draw":
			draw();
			break;
		case "setBrushSize":
			setBrushSize(message.data);
			break;
		case "setBrushColour":
			setBrushColour(message.data);
			break;
		case "setErasing":
			setErasing(message.data);
			break;
		case "clearCanvas":
			clearCanvas();
			break;
		default:
			console.log("Unknown message type: " + message.type);
			break;
	}
}

function setRole(newRole)
{
	role = newRole;

	switch(role)
	{
		case "drawer":
			canvas.style.cursor = drawingCursor;
			drawingToolbox.style.display = "";
			guessInput.placeholder = "Type your message here...";
			break;
		case "guesser":
			canvas.style.cursor = "";
			drawingToolbox.style.display = "none";
			guessInput.placeholder = "Type your guess here...";
			break;
		case "waiter":
			canvas.style.cursor = "";
			drawingToolbox.style.display = "none";
			guessInput.placeholder = "Type your message here...";
			break;
		case "wordFinder":
			canvas.style.cursor = "";
			drawingToolbox.style.display = "none";
			guessInput.placeholder = "Type your message here...";
			break;
		default:
			console.log("Unknown role: " + role);
			break;
	}
}

function makeGuess(text)
{
	sendMessage("makeGuess", text);
}

function sendChatMessage(text)
{
	sendMessage("sendChatMessage", text);
}

function addGuessElement(guesser, text)
{
	let element = document.createElement("p");
	element.style.marginTop = "0";
	element.style.marginBottom = "0";
	element.innerHTML = "<b>" + guesser + ": </b>" + text;
	guessingBox.appendChild(element);
}

function addChatMessageElement(sender, text)
{
	let element = document.createElement("p");
	element.style.marginTop = "0";
	element.style.marginBottom = "0";
	element.style.color = "green";
	element.innerHTML = "<b>" + sender + ": </b>" + text;
	guessingBox.appendChild(element);
}

function revealWord(word)
{
	let element = document.createElement("p");
	element.style.marginTop = "0";
	element.style.marginBottom = "0";
	element.style.color = "green";
	element.innerHTML = "The word was <b>" + word + "</b>";
	guessingBox.appendChild(element);

	endTimer();
}

function setChoosingPlayer(drawer)
{
	setRole("guesser");

	let element = document.createElement("p");
	element.style.marginTop = "0";
	element.style.marginBottom = "0";
	element.style.color = "blue";
	element.innerHTML = "<b>" + drawer + "</b> is choosing a word";
	guessingBox.appendChild(element);
}

function addNewDrawerElement(drawer)
{
	let element = document.createElement("p");
	element.style.marginTop = "0";
	element.style.marginBottom = "0";
	element.style.color = "blue";
	element.innerHTML = "<b>" + drawer + "</b> is now drawing!";
	guessingBox.appendChild(element);
}

function addPlayerJoinElement(player)
{
	let element = document.createElement("p");
	element.style.marginTop = "0";
	element.style.marginBottom = "0";
	element.style.color = "green";
	element.innerHTML = "<b>" + player + "</b> joined";
	guessingBox.appendChild(element);
}

function addPlayerLeaveElement(player)
{
	let element = document.createElement("p");
	element.style.marginTop = "0";
	element.style.marginBottom = "0";
	element.style.color = "red";
	element.innerHTML = "<b>" + player + "</b> left";
	guessingBox.appendChild(element);
}

function addNewWordFinderElement(drawer)
{
	let element = document.createElement("p");
	element.style.marginTop = "0";
	element.style.marginBottom = "0";
	element.style.color = "green";
	element.innerHTML = "<b>" + drawer+ "</b> guessed the word!";
	guessingBox.appendChild(element);
}

function setWordOptions(words)
{
	wordOptions = words;

	firstWordButton.innerText = words[0];
	secondWordButton.innerText = words[1];
	thirdWordButton.innerText = words[2];

	wordChoiceDiv.style.display = "";
}

function setStartPoint(x, y)
{
	startPoint = { x: x, y: y };

	if(role == "drawer")
	{
		sendMessage("setStartPoint", startPoint);
	}
}

function setEndPoint(x, y)
{
	endPoint = { x: x, y: y };

	if(role == "drawer")
	{
		sendMessage("setEndPoint", endPoint);
	}
}

function draw()
{
	//TODO: Check if within bounds

	let colour = (erasing ? "#FFFFFF" : brush.colour);
	let size = brush.size;

	context.strokeStyle = colour;
	context.lineWidth = size;

	context.beginPath();
	context.moveTo(startPoint.x, startPoint.y);
	context.lineTo(endPoint.x, endPoint.y);
	context.closePath();
	context.stroke();

	if(role == "drawer")
	{
		sendMessage("draw", null);
	}
}

function startTimer()
{
	timerSpan.innerText--; //HACK: Do some proper time syncing so we don't have to do this
	timerId = window.setInterval(function() {
		if(timerSpan.innerText != 0)
		{
			timerSpan.innerText--;
		}
	}, 1000);
}

function endTimer()
{
	clearInterval(timerId);
}

function setErasing(state)
{
	erasing = state;
	eraserToggle.innerText = (erasing ? "Eraser" : "Brush");

	if(role == "drawer")
	{
		sendMessage("setErasing", erasing);
	}
}

function clearCanvas()
{
	context.clearRect(0, 0, canvas.width, canvas.height);

	if(role == "drawer")
	{
		sendMessage("clearCanvas", null);
	}
}

function setBrushSize(size)
{
	brush.size = size;

	if(role == "drawer")
	{
		sendMessage("setBrushSize", size);
	}
}

function setBrushColour(colour)
{
	brush.colour = colour;

	if(role == "drawer")
	{
		sendMessage("setBrushColour", colour);
	}
}

canvas.onmousedown = function(e) {
	if(role != "drawer")
	{
		return;
	}

	drawing = true;

	setStartPoint(e.clientX - canvas.offsetLeft, e.clientY - canvas.offsetTop);
	setEndPoint(e.clientX - canvas.offsetLeft + 1, e.clientY - canvas.offsetTop);
	draw();
	setStartPoint(endPoint.x, endPoint.y);
}

canvas.onmousemove = function(e) {
	if(role != "drawer" || !drawing)
	{
		return;
	}

	setEndPoint(e.clientX - canvas.offsetLeft, e.clientY - canvas.offsetTop);
	draw();
	setStartPoint(endPoint.x, endPoint.y);
}

//TODO: Stop drawing on mouseup outside canvas
canvas.onmouseup = function(e) {
	drawing = false;
}

eraserToggle.onclick = function(e) {
	setErasing(!erasing);
}

clearButton.onclick = function(e) {
	clearCanvas();
}

toolSizeSelect.onchange = function(e) {
	setBrushSize(toolSizeSelect.value);
}

colourSelect.onchange = function(e) {
	setBrushColour(colourSelect.value);
}

guessInput.onkeydown = function(e) {
	if(e.keyCode == 13) //enter key keycode
	{
		if(role == "guesser")
		{
			makeGuess(guessInput.value);
		}
		else
		{
			sendChatMessage(guessInput.value);
		}
		guessInput.value = "";
	}
}

firstWordButton.onclick = function(e) {
	sendMessage("chooseWord", wordOptions[0]);
}

secondWordButton.onclick = function(e) {
	sendMessage("chooseWord", wordOptions[1]);
}

thirdWordButton.onclick = function(e) {
	sendMessage("chooseWord", wordOptions[2]);
}

init();
